#!/usr/bin/perl

########################################################################
# metaf2xml.pl 1.42
#   example script to parse and print METAR, TAF, and SYNOP messages
#
# copyright (c) metaf2xml 2006-2010
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
########################################################################

########################################################################
# some things strictly perl
########################################################################
use strict;
use warnings;

=head1 NAME

metaf2xml.pl

=head1 SYNOPSIS

  metaf2xml.pl [OPTION]... [MESSAGE]...

=head1 DESCRIPTION

This script is an example interface to the Perl modules C<metaf2xml::parser>
and C<metaf2xml::XML>. Messages can be specified on the command line (in single
or double quotes).
If no messages are specified and the option -f is provided, the specified file
is read, otherwise the standard input is read.

Messages are required to be delimited by a newline and
in the format specified by the I<WMO Manual No. 306>, B<without modifications
due to distribution> (which e.g. lists the initial part of messages only once
for several messages, or uses the "=" (equal sign) as a delimiter),
and parts of the WMO header prepended according to the option -H.
The program C<metafsrc2raw.pl> can be used to create messages with the required
format from files provided by various public Internet servers.

The initial default message type is METAR. A message consisting only of one of
the keywords C<METAR>, C<SPECI>, C<TAF>, or C<SYNOP> changes the default
message type for all following messages. If a message starts with one of these
keywords, the default message type is ignored for this message.
If the option -H1, -H2, -H3, or -H4 is given, the message type for each message
is taken from the WMO header.

=head1 DEPENDENCIES

The Perl modules C<Getopt::Std>, C<metaf2xml::parser>, and C<metaf2xml::XML> are
required.

=head1 OPTIONS

=over

=item -f I<msg_file>

read input from I<msg_file>

=item -x I<xml_file>

enables writing the data to I<xml_file>

=item -H I<0..4>

which parts of the WMO header (C<TTAA[II] CCCC DDHHMM> [modifiers]) are
prepended to each message (see the manual for C<metafsrc2raw.pl> for an
explanation of its parts):

=over

=item -H0 (default)

(no header)

=item -H1

C<TT> report type

=item -H2

C<TTAA[II]> (like C<-H1> plus region for the stations and optional report number)

=item -H3

C<TTAA[II] CCCC> (like C<-H2> plus report dissimination location)

=item -H4

C<TTAA[II] CCCC DDHHMM> (like C<-H3> plus report dissimination time)

=back

=item -T {C<SPECI>|C<TAF>|C<SYNOP>}

(initial) default message type (default: C<METAR>)

=item -D

include DOCTYPE and reference to the DTD

=item -S I<xslt_file>

include reference to the stylesheet I<xslt_file>

=item -O I<options>

include I<options> (a space separated list of options)

=back

Without the option -x, no output is generated.

=head1 EXAMPLES

Parse a METAR message and write the XML section C<<reports>> to the standard
output:

  metaf2xml.pl -x- 'YUDO 090600Z 00000KT CAVOK 22/15 Q1021 NOSIG'

The same for a SYNOP message:

  metaf2xml.pl -TSYNOP -x- 'AAXX 17124 74486 32566 63616 10067 20022 39972'

or:

  metaf2xml.pl -x- 'SYNOP AAXX 17124 74486 32566 63616 10067 20022 39972'

Parse several METAR, TAF, and SYNOP messages and write the XML to the file
F<metar-taf-synop.xml>:

  metaf2xml.pl -x metar-taf-synop.xml <<EOF
METAR
SBGL 091800Z 14008KT 9999 FEW020 BKN035 23/15 Q1013
TAF
SBGL 091550Z 091818 20015KT 8000 BKN020 PROB40 2024 4000 RA BKN015 BECMG 0002 24005KT BKN015 TEMPO 0210 4000 RA BR BKN010 BECMG 1113 20010KT 4000 RA BR TX22/19Z TN18/08Z
METAR
KJFK 091751Z 34013KT 10SM SCT038 21/11 A2952
TAF
KJFK 091425Z 091412 34012KT P6SM OVC025 TEMPO 1416 SCT025 FM1600 32013G18KT P6SM SCT025 BKN040 FM1800 31010KT P6SM SCT050 FM2200 28010KT P6SM SKC
METAR
RJTT 091830Z 34005KT CAVOK 14/09 Q1020 RMK A3012
TAF
RJTT 091500Z 091524 30004KT 9999 FEW020
RJTT 091500Z 100018 05005KT 9999 FEW030 BECMG 0003 14006KT BECMG 0912 30010KT
SYNOP
AAXX 17124 83746 21558 33602 10263 20218 40171 51014 70596 82530 333 20227 69924
AAXX 17124 74486 12566 63616 10067 20022 39972 49982 51028 69931 91151 333 10128 20056 70025 91021 555 91712
EOF

=head1 SEE ALSO

=begin html

<a href=parser.pm.html>metaf2xml::parser(3pm)</a>,
<a href=XML.pm.html>metaf2xml::XML(3pm)</a>,
<a href=metafsrc2raw.pl.html>metafsrc2raw(1)</a>,
<a href=metaf.pl.html>metaf(1)</a>, <!--

=end html

B<metaf2xml::parser>(3pm),
B<metaf2xml::XML>(3pm),
B<metafsrc2raw>(1),
B<metaf>(1),

=for html -->

L<http://metaf2xml.sourceforge.net/>

=head1 COPYRIGHT and LICENSE

copyright (c) 2006-2010 metaf2xml @ L<http://metaf2xml.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA

=cut

use Getopt::Std;

# path may be changed by install
use lib '/usr/local/lib/perl5/site_perl/5.14'; # METAF2XML_LIB
use metaf2xml::parser 1.42 qw(parse_report);
use metaf2xml::XML 1.42 qw(start_xml print_report finish_xml);

{
    my $default_msg_type = 'METAR';

    sub go {
        my ($msg, $do_print, $opts_h) = @_;
        my (%report_data, $msg_type, %WMOheader);

        if ($msg =~ /^(?:METAR|SPECI|TAF|SYNOP)$/) {
            $default_msg_type = $msg;
            return;
        }

        $msg_type = $default_msg_type;
        if ($opts_h > 0) {
            $msg =~ s/^(..)//;
            $WMOheader{TT} = $1;
            if ($opts_h >= 2) {
                $msg =~ s/^([^ ]+)//;
                $WMOheader{AAII} = $1;
            }
            if ($opts_h >= 3) {
                $msg =~ s/^ ([^ ]+)//;
                $WMOheader{CCCC} = $1;
            }
            if ($opts_h == 4) {
                $msg =~ s/^ (\d\d)(\d\d)(\d\d)//;
                @{\%WMOheader}{qw(day hour minute)} = ($1, $2, $3);
            }
            $msg =~ s/^ //;
            $msg_type = { SA => 'METAR',
                          SP => 'SPECI',
                          FC => 'TAF',
                          FT => 'TAF',
                          SM => 'SYNOP',
                          SI => 'SYNOP',
                          SN => 'SYNOP'
                        }->{$WMOheader{TT}}
                if $WMOheader{TT} =~ /^(?:S[APMIN]|F[CT])$/;
        }

        %report_data = parse_report $msg, $msg_type;
        # $report_data{WMOheader} = \%WMOheader if exists $WMOheader{TT};
        print_report \%report_data if $do_print;
    }
}

sub usage {
    print STDERR
          (shift)
        . "Usage: metaf2xml.pl [OPTION]... [MESSAGE]...\n"
        . "  Options:\n"
        . "  -f <msg_file>  read input from <msg_file>\n"
        . "  -x <xml_file>  enables writing the data to <xml_file>\n"
        . "  -H <0..4>      which parts of the WMO header are prepended\n"
        . "  -T <msg_type>  (initial) default message type: SPECI, TAF, SYNOP\n"
        . "  -D             include DOCTYPE and reference to the DTD\n"
        . "  -S <xslt_file> include reference to the stylesheet <xslt_file>\n"
        . "  -O <options>   include <options> (a space separated list of options)\n"
        . "Without option -x, no output is generated.\n";
    exit 1;
}

sub main {
    my (%opts, $do_print);

    # get and check options
    usage '' unless getopts('f:x:H:T:DS:O:', \%opts);
    $opts{H} = 0 unless exists $opts{H} && $opts{H} =~ /^[0-4]$/;

    # set initial state for message type
    if (exists $opts{T}) {
        if ($opts{T} =~ /^(?:SPECI|TAF|SYNOP)$/) {
            go $opts{T};
        } else {
            usage "ERROR: invalid message type: '" . $opts{T} . "'.\n";
        }
    }

    $do_print = exists $opts{x};

    start_xml \%opts
        if $do_print;

    if ($#ARGV >= 0) {
        for (@ARGV) {
            for (split /\n/, $_) {
                go $_, $do_print, $opts{H};
            }
        }
    } elsif (exists $opts{f}) {
        open(FP, '<', $opts{f}) || usage "ERROR: cannot open '$opts{f}'\n";
        while (<FP>) {
            chomp();
            go $_, $do_print, $opts{H};
        }
        close FP;
    } else {
        while (<STDIN>) {
            chomp();
            go $_, $do_print, $opts{H};
        }
    }

    finish_xml;
}

&main;
